# Gauss-Seidel-Verfahren/SOR-Verfahren zur Loesung der
import time, sys

'''
 Gauss-Seidel/SOR-Verfahren zur Loesung der 2D Randwertaufgabe
       -delta u(x,y) = f(x,y) im Innern des Einheitsquadrates
              u(x,y) = g(x,y) auf dem Rand des Einheitsquadrats
'''

def frhs(x,y):
   # rechte Seite f(x,y) der Randwertaufgabe
   #f = -6.
   f = -12.*x**2-24.*y**2
   return f
def urhs(x,y):
   # Randwerte g(x,y) der Randwertaufgabe
   #u = x*x + 2.*y*y
   u = x**4 + 2.*y**4
   return u
def gs_sor_iteration(u,f,h2,omega,n):
   for i in range (1,n-1):
      for j in range (1,n-1):
         tmp=0.25*(f[i][j]*h2+u[i-1][j]+u[i][j-1]+u[i+1][j]+u[i][j+1])
         u[i][j]=u[i][j]+omega*(tmp-u[i][j])
   return u
def resicalc(u,f,h2,n):
   resi=0.
   for i in range (1,n-1):
      for j in range (1,n-1):
         resi = max(resi,abs(f[i][j]\
               +(u[i-1][j]+u[i][j-1]+u[i+1][j]+u[i][j+1]-4.0*u[i][j])/h2))
   return resi
def max_err(u,h,n):
   maxerr=0.
   for i in range(n):
      x = i * h
      for j in range(n):
         y=j*h
         maxerr = max(maxerr,abs(u[i][j]-urhs(x,y)))
   return maxerr

# Hauptprogramm zur Loesung der
# 2D Randwertaufgabe
#       -delta u = f 
#              u = g
# im Einheitsquadrat
# mit dem Gauss-Seidel-Verfahren/SOR-Verfahren
#
# Eingabeparameter:
# nint : Anzahl der Intervalle in x- und y-Richtung
# niter: Maximale Anzahl an Iterationsschritten
# epsilon: Genauigkeit, mit der das GLS geloest werden soll
#          die Iterationen sollen abbrechen, wenn der Defekt/das
#          Residuum kleiner oder gleich epsilon geworden ist.
# omega: Relaxationsparameter (0 < omega < 2); fuer omega=1 erhaelt man
#        das Gauss-Seidel-Verfahren
#
print
print "Programm zur Loesung der 2D Poissonschen Randwertaufgabe im Einheitsquadrat"
print "mit dem SOR-/Gauss-Seidel-Verfahren"
print
nint = input("Anzahl der Intervalle in x- und y-Richtung: ")
n = nint+1
niter = input("Maximale Anzahl von Iterationsschritten: ")
epsilon = float(input("Schranke fuer das Residuum; Verfahren stoppt, wenn diese unterschritten wird: "))
omega = float(input("Relaxationsparameter omega (0 < omega < 2); omega=1 liefert Gauss-Seidel-Verfahren: "))
time0=time.time()
h=1./nint
h2=h*h
x=[]
y=[]
for i in range (0,n):
   x.append(i*h)
   y.append(i*h)
f1=range(n)
f=range(n)
u=range(n)
for i in range (0,n):
   f[i]=f1[:]
   u[i]=f1[:]
for i in range (1,n-1):
   for j in range (1,n-1):
      f[i][j]=frhs(x[i],y[j])
for i in range (0,n):
   for j in range (0,n):
      u[i][j]=urhs(x[i],y[j])
for i in range (1,n-1):
   for j in range (1,n-1):
      u[i][j]=0.
resi=resicalc(u,f,h2,n)
print 'Startresiduum =', resi
for iter in range (1,niter+1):
   u = gs_sor_iteration(u,f,h2,omega,n)
   resi=resicalc(u,f,h2,n)
   if resi <= epsilon or iter == niter or iter%10 == 0:
      maxerr = max_err(u,h,n)
      print 'Iterationschritt Nr. %5i, Residuum = %.13f, Max. Fehler = %.12f'%(iter,resi,maxerr)
   if resi <= epsilon:
      break
time1=time.time()
print "Benoetigte Zeit: ", time1-time0
sys.stdin.readline()
